(*
   Icon Viewer
   Version 6.00
   (C) 1996, Glen Why

   Main Form Class

*)
unit ICOVIEW1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ShellAPI, Clipbrd, ComCtrls, ExtCtrls, Menus, Buttons,
  DriveList, AnotherReg, FileIterator, IcoView2;

const

  PANEL_HINT = 0;
  PANEL_COUNT = 1;
  PANEL_DRIVE = 2;
  PANEL_DETAILS = 3;

type

  TMainForm = class(TForm)
    StatusBar: TStatusBar;
    List: TListView;
    MainMenu: TMainMenu;
    FileItem: TMenuItem;
    FileExitItem: TMenuItem;
    DriveList: TDriveList;
    AppConfigKey: TAppConfigKey;
    FormRestorer1: TFormRestorer;
    ViewItem: TMenuItem;
    ViewRefreshItem: TMenuItem;
    Iterator: TFileIterator;
    N1: TMenuItem;
    ViewIconsItem: TMenuItem;
    ViewDetailsItem: TMenuItem;
    Icons: TImageList;
    ViewSmallIconsItem: TMenuItem;
    SmallIcons: TImageList;
    ButtonPanel: TPanel;
    N2: TMenuItem;
    FileSaveItem: TMenuItem;
    EditItem: TMenuItem;
    EditCopyItem: TMenuItem;
    N3: TMenuItem;
    EditSelectAllItem: TMenuItem;
    EditInvertSelectionItem: TMenuItem;
    ViewAsListItem: TMenuItem;
    DoneScanEvent: TSoundEvent;
    SaveEvent: TSoundEvent;
    CopyEvent: TSoundEvent;
    SelectAllEvent: TSoundEvent;
    Options: TRegKey;
    ViewStyle: TVariantProfile;
    ListPopupMenu: TPopupMenu;
    ppViewItem: TMenuItem;
    ppLargeIcons: TMenuItem;
    ppSmallIcons: TMenuItem;
    ppList: TMenuItem;
    ppDetails: TMenuItem;
    ppRefresh: TMenuItem;
    N4: TMenuItem;
    ppCopy: TMenuItem;
    N5: TMenuItem;
    ppSelectAll: TMenuItem;
    ppInvertSelection: TMenuItem;
    N6: TMenuItem;
    ppSave: TMenuItem;
    SaveDialog: TSaveDialog;
    Help1: TMenuItem;
    AboutItem: TMenuItem;
    procedure UpdateFileMenu(Sender: TObject);
    procedure CloseApp(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure UpdateViewMenu(Sender: TObject);
    procedure Refresh(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure IteratorActivate(Sender: TObject);
    procedure IteratorTerminate(Sender: TObject);
    procedure IteratorAddFileQuery(Sender: TObject; const FileName: string;
      const FileInfo: TFileInfo; var CanAdd: Boolean);
    procedure IteratorAddFile(Sender: TObject; const FileName: string;
      const FileInfo: TFileInfo);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ViewAsIcons(Sender: TObject);
    procedure ViewDetails(Sender: TObject);
    procedure ViewAsSmallIcons(Sender: TObject);
    procedure SaveFile(Sender: TObject);
    procedure UpdateEditMenu(Sender: TObject);
    procedure CopyFile(Sender: TObject);
    procedure SelectAll(Sender: TObject);
    procedure InvertSelection(Sender: TObject);
    procedure ViewAsList(Sender: TObject);
    procedure ListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure UpdatePopupMenu(Sender: TObject);
    procedure UpdatePPViewMenu(Sender: TObject);
    procedure ABout(Sender: TObject);
  private
    FIconCount :Integer;
    FDrive :String;
    FFileList :TStringList;
    FOldViewStyle :TViewStyle;
    FTempIcon :TIcon;
    FSaveData :TSaveData;
    function GetStatusPanel( index :integer ):TStatusPanel;
    procedure ShowHint( Sender :TObject );
    procedure SetCount( Value :Integer );
    procedure SetDetails( const Value :String );
    function GetDetails :string;
    procedure CreateDriveButtons;
    procedure DriveButtonClick( Sender :TObject );
    procedure UpdateDrivePanel;
  protected
    procedure UpdateControls;
    property HintPanel :TStatusPanel index PANEL_HINT read GetStatusPanel;
    property CountPanel :TStatusPanel index PANEL_COUNT read GetStatusPanel;
    property DetailsPanel :TStatusPanel index PANEL_DETAILS read GetStatusPanel;
    property DrivePanel :TStatusPanel index PANEL_DRIVE read GetStatusPanel;
    property Details :string read GetDetails write SetDetails;
    property IconCount :Integer read FIconCount write SetCount;
  end;

var
  MainForm: TMainForm;

implementation

uses ICOVIEW3;

{$R *.DFM}

{$include IcoView.inc}

const

IconFiles = '.DLL.EXE.ICO.ANI.SCR.CPL.ICL.OCX.VBX';

IconSmallSize = 16;
IconNormalSize = 32;
IconLargeSize = 64;

procedure TMainForm.UpdateFileMenu(Sender: TObject);
begin
  FileSaveItem.Enabled := List.SelCount > 0;
end;

procedure TMainForm.CloseApp(Sender: TObject);
begin
  Close;
end;

function TMainForm.GetStatusPanel( index :integer ):TStatusPanel;
begin
  result := StatusBar.Panels[ index ];
end;

procedure TMainForm.ShowHint( Sender :TObject );
begin
  HintPanel.Text := Application.Hint;
end;

procedure TMainForm.CreateDriveButtons;
var I, B, X, Y :Integer;
begin
  X := 2; Y := 5;
  with DriveList do
   begin
    for I := 0 to DriveCount - 1 do
      with TSpeedButton.Create( ButtonPanel ) do
        begin
          Caption := Drives[ i ];
          GroupIndex := 1;
          AllowAllUp := false;
          Margin := 0;
          Spacing := 2;
          OnClick := DriveButtonClick;
          case DriveType[ i ] of
            dtFixed, dtRAM : B := BMP_FIXED;
            dtRemote : B := BMP_REMOTE;
            dtRemovable: B := BMP_FLOPPY;
            dtCDROM : B := BMP_CDROM;
            else B := BMP_FIXED;
          end;
          Glyph := TBitmap.Create;
          Glyph.Handle := LoadBitmap( hInstance, PChar( B ) );
          Width := Canvas.TextWidth( Caption ) + Glyph.Width + 10;
          Height := Glyph.Height + 4;
          Left := X; Top := Y;
          inc( X, Width );
          Parent := ButtonPanel;
          Visible := true;
        end;
   end;
end;


procedure TMainForm.FormCreate(Sender: TObject);
begin
  CreateDriveButtons;
  HintPanel.Text := '';
  Application.OnHint := ShowHint;
  IconCount := 0;
  Details := '';
  if Options.Exist then
   begin
     List.ViewStyle := ViewStyle.Value;
   end;
  UpdateDrivePanel;
end;

procedure TMainForm.SetCount( Value :Integer );
begin
  FIconCount := Value;
  CountPanel.Text := format( '%d icon(s)', [ value ] );
end;

procedure TMainForm.SetDetails( const Value :String );
begin
  DetailsPanel.Text := Value;
end;

function TMainForm.GetDetails :string;
begin
  result := DetailsPanel.Text;
end;

procedure TMainForm.DriveButtonClick( Sender :TObject );
begin
  FDrive := ( Sender as TSpeedButton ).Caption;
  UpdateDrivePanel;
end;

procedure TMainForm.UpdateDrivePanel;
begin
 if FDrive = ''
   then DrivePanel.Text := 'No Drive'
   else DrivePanel.Text := FDrive;
end;

procedure TMainForm.UpdateViewMenu(Sender: TObject);
begin
  ViewRefreshItem.Enabled := Iterator.IsIdle;
  case List.ViewStyle of
    vsIcon : ViewIconsItem.Checked := true;
    vsReport : ViewDetailsItem.Checked := true;
    vsSmallIcon : ViewSmallIconsItem.Checked := true;
    vsList : ViewAsListItem.Checked := true;
  end;
end;

procedure TMainForm.UpdateControls;
begin
 UpdateViewMenu( Self );
 UpdateFileMenu( Self );
 UpdateEditMenu( Self );
end;


procedure TMainForm.Refresh( Sender: TObject );
begin
  if not Iterator.IsIdle then exit;
  if ( FDrive = '' ) then raise Exception.CreateRes( SNoDrive );
  Iterator.RootFolder := FDrive;
  TFileIteratorThread.Create( Iterator, false );
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
 if not Iterator.IsIdle then
   begin
     CanClose := false;
     ShowMessage( LoadStr( SCloseError ) );
   end;
end;

procedure TMainForm.IteratorActivate(Sender: TObject);
begin
  Details := LoadStr( SWormingUp );
  IconCount := 0;
   with List do begin
     Visible := false;
     Items.BeginUpdate;
     Items.Clear;
     FOldViewStyle := ViewStyle;
     ViewStyle := vsList;
   end;
  Icons.Clear;
  SmallIcons.Clear;
  FTempIcon := TIcon.Create;
  UpdateControls;
  Details := LoadStr( SSearching );
end;

procedure TMainForm.IteratorTerminate(Sender: TObject);
begin
  Details := LoadStr( STerminating );
  DoneScanEvent.Play;
  FTempIcon.Free;
  with List do begin
    ViewStyle := FOldViewStyle;
    Items.EndUpdate;
    Visible := true;
  end;
  UpdateControls;
  Details := LoadStr( SIdle );
end;

procedure TMainForm.IteratorAddFileQuery(Sender: TObject;
  const FileName: string; const FileInfo: TFileInfo; var CanAdd: Boolean);
begin
  CanAdd := Pos( UpperCase( ExtractFileExt( FileName ) ), IconFiles ) > 0;
end;

procedure TMainForm.IteratorAddFile(Sender: TObject;
  const FileName: string; const FileInfo: TFileInfo);
var
 C, I :Integer;
 H :hIcon;
begin
 C := ExtractIcon( hInstance, PChar( FileName ), -1 );
 if ( C > 0 )
   then begin
     for I := 0 to C - 1
       do begin
         H := ExtractIcon( hInstance, PChar( FileName ), I );
         if ( H <= 1 ) then Continue;
         FTempIcon.Handle := H;
         SmallIcons.AddIcon( FTempIcon );
         with List.Items.Add do begin
           ImageIndex := Icons.AddIcon( FTempIcon );
           StateIndex := -1;
           SubItems.Add( IntToStr( I ) );
           Caption := ExtractFileName( FileName );
         end;
       end;
     IconCount := IconCount + C;
   end;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then
    if Iterator.IsIdle
      then close
      else begin
        Details := LoadStr( STerminating );
        Iterator.Cancel;
      end;
end;

procedure TMainForm.ViewAsIcons(Sender: TObject);
begin
   if Iterator.IsIdle then List.ViewStyle := vsIcon;
end;

procedure TMainForm.ViewDetails(Sender: TObject);
begin
   if Iterator.IsIdle then List.ViewStyle := vsReport;
end;

procedure TMainForm.ViewAsSmallIcons(Sender: TObject);
begin
   if Iterator.IsIdle then List.ViewStyle := vsSmallIcon;
end;

procedure TMainForm.UpdateEditMenu(Sender: TObject);
begin
  EditCopyItem.Enabled := assigned( List.ItemFocused );
  EditSelectAllItem.Enabled := List.Items.Count > 0;
  EditInvertSelectionItem.Enabled := EditSelectAllItem.Enabled;
end;

procedure TMainForm.CopyFile(Sender: TObject);
var
  Item :TListItem;
  Bmp :TBitmap;
begin
  CopyEvent.Play;
  Item := List.ItemFocused;
  if ( item = nil ) then exit;
  Bmp := TBitmap.Create;
  try
    Icons.GetBitmap( Item.ImageIndex, Bmp );
    Clipboard.Assign( Bmp );
  finally
    Bmp.Free;
  end;
end;

procedure TMainForm.SelectAll(Sender: TObject);
var i :integer;
begin
   SelectAllEvent.Play;
   with List.Items do begin
     BeginUpdate;
     for i := 0 to Count - 1 do Item[ i ].Selected := true;
     EndUpdate;
   end;
end;

procedure TMainForm.InvertSelection(Sender: TObject);
var i :integer;
begin
   with List.Items do begin
     BeginUpdate;
     for i := 0 to Count - 1
      do with Item[ i ] do Selected := not Selected;
     EndUpdate;
   end;
end;

procedure TMainForm.ViewAsList(Sender: TObject);
begin
  if Iterator.IsIdle then List.ViewStyle := vsList;
end;

procedure TMainForm.ListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  if Iterator.IsIdle then UpdateControls;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ViewStyle.Value := List.ViewStyle;
end;

procedure TMainForm.UpdatePopupMenu(Sender: TObject);
begin
  ppRefresh.Enabled := Iterator.IsIdle;
  ppCopy.Enabled := assigned( List.ItemFocused );
  ppSelectAll.Enabled := List.Items.Count > 0;
  ppInvertSelection.Enabled := ppSelectAll.Enabled;
  ppSave.Enabled := List.SelCount > 0;
end;

procedure TMainForm.UpdatePPViewMenu(Sender: TObject);
begin
  case List.ViewStyle of
    vsIcon : ppLargeIcons.Checked := true;
    vsReport : ppDetails.Checked := true;
    vsSmallIcon : ppSmallIcons.Checked := true;
    vsList : ppList.Checked := true;
  end;
end;

procedure TMainForm.SaveFile(Sender: TObject);
var
  N :String;
  I :Integer;
  G :TGraphic;
begin
 if List.SelCount <= 0 then exit;
 if not GetSaveData( Self, FSaveData ) then raise EAbort.Create('');
 G := Nil;
 Details := LoadStr( SSaving );
 StatusBar.Repaint;
 try
  with List.Items do
   for i := 0 to Count - 1 do
     with Item[ i ] do
      if Selected then
        begin
          case FSaveData.ImageType of
            itIcon :
              begin
                G := TIcon.Create;
                Icons.GetIcon( ImageIndex, TIcon( G ) );
              end;
            itSmallIcon :
              begin
                G := TIcon.Create;
                SmallIcons.GetIcon( ImageIndex, TIcon( G ) );
              end;
            iTBitmap :
              begin
                G := TBitmap.Create;
                Icons.GetBitmap( ImageIndex, TBitmap( G ) );
              end;
            itSmallBitmap :
              begin
                G := TBitmap.Create;
                SmallIcons.GetBitmap( ImageIndex, TBitmap( G ) );
              end;
          end;
          try
            N := Caption;
            N := ChangeFileExt( N, '' ) + SubItems.Strings[ 0 ];
            if ( G is TBitmap )
              then N := ChangeFileExt( N, '.bmp' )
              else N := ChangeFileExt( N, '.ico' );
            while FileExists( N ) do N := '_' + N;
            if FSaveData.NamingMode = nmPrompt
              then with SaveDialog do begin
                FIleName := N;
                if not Execute then raise EAbort.Create('');
                N := FileName;
              end;
            try
              G.SaveToFile( N );
            except
              ShowMessage( Format( LoadStr( SSaveError ), [ N ] ) );
            end;
          finally
            G.Free;
          end;
        end;
  SaveEvent.Play;
 finally
   Details := LoadStr( SIdle );
 end;
end;





procedure TMainForm.ABout(Sender: TObject);
begin
 AboutBox( Self );
end;

end.


